/*
Stolyarevskiy Sergey, acobw.narod.ru

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "main.h"
HINSTANCE g_hInst;

BOOL APIENTRY DllMain (HINSTANCE hInst     /* Library instance handle. */ ,
                       DWORD reason        /* Reason this function is being called. */ ,
                       LPVOID reserved     /* Not used. */ )
{
    switch (reason)
    {
        case DLL_PROCESS_ATTACH:
        //  hInst
        g_hInst=hInst;
        break;
        case DLL_PROCESS_DETACH:
        break;
        case DLL_THREAD_ATTACH:
        break;
        case DLL_THREAD_DETACH:
        break;
    }
    /* Returns TRUE on success, FALSE on failure */
    return TRUE;
}

VDXScriptFunctionDef script_functions[] =
    {
        {
            (VDXScriptFunctionPtr)script_config, "Config", "0iiiii"
        },
        {
            NULL, NULL, NULL
        },
    };
VDXScriptObject script_obj =
    {
        NULL, script_functions, NULL
    };

static struct VDXFilterDefinition g_myFilterDef=
    {
        0,0, NULL,       // next, prev, module
        "Frequency Suppressor of the Noise 3.6.1 MT",             // name
        "Frequency Suppressor of the Noise, version 3.6.1 MT",     // desc
        "Stolyarevskiy Sergey", // maker
        NULL,                   // private_data
        sizeof(MyFilterData),   // inst_data_size
        InitProc,               // initProc
        NULL,                   // deinitProc
        RunProc,				// runProc
        NULL,				    // paramProc
        configProc,				// configProc
        StringProc,				// stringProc
        StartProc,				// startProc
        EndProc,				// endProc
        &script_obj,			// script_obj
        FssProc,				// fssProc
        NULL,
        NULL,
        NULL,
    };

extern VDXFilterDefinition g_myFilterDef;
VDXFilterDefinition *g_registeredFilterDef;
///////////////////////////////////////////////////////////////////////////

extern "C" __declspec(dllexport) int __cdecl VirtualdubFilterModuleInit2(struct VDXFilterModule *fm, const VDXFilterFunctions *ff, int& vdfd_ver, int& vdfd_compat)
{
    g_registeredFilterDef = ff->addFilter(fm, &g_myFilterDef, sizeof(VDXFilterDefinition));

    vdfd_ver        = VIRTUALDUB_FILTERDEF_VERSION;
    vdfd_compat     = 8;    // we need this version for copy constructor support

    return 0;
}

extern "C" __declspec(dllexport) void __cdecl VirtualdubFilterModuleDeinit(struct VDXFilterModule *fm, const VDXFilterFunctions *ff)
{
    // Note: This must be the pointer returned from addFilter(), NOT your original
    // declaration.
    ff->removeFilter(g_registeredFilterDef);
}

int InitProc(VDXFilterActivation *fa, const VDXFilterFunctions *ff)
{
    //   
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    mfd->Disable=0;	//  
    mfd->Noise=0;		//   
    mfd->Sharpen=2;		//     
    mfd->Level_1=70;	//    1  
    mfd->Level_2=70;	//    2  
    mfd->interlace=0;	//   ?
    SYSTEM_INFO siSysInfo;
    GetSystemInfo(&siSysInfo);
    mfd->count=siSysInfo.dwNumberOfProcessors;	//    
    if (mfd->count>4)
        mfd->count=4; //    4  (  = -  )
    if (mfd->count>1) //         
    {
        mfd->speed=1;
    }
    else
    {
        mfd->speed=0;
    }
    return 0;
}

INT_PTR CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    MyFilterData *mfd = (MyFilterData *)GetWindowLongPtr(hdlg, DWLP_USER);
    switch (msg)
    {

        case WM_INITDIALOG:
        SetWindowLongPtr(hdlg, DWLP_USER, (LONG_PTR)lParam);
        mfd = (MyFilterData *)lParam;

        SendDlgItemMessage (hdlg, IDC_SLIDER_LF1, TBM_SETTICFREQ, 0, 0);
        SendDlgItemMessage (hdlg, IDC_SLIDER_LF1, TBM_SETRANGE, false, MAKELONG (0, 100));
        SendDlgItemMessage (hdlg, IDC_SLIDER_LF1, TBM_SETPOS, true, mfd->Level_1);

        SendDlgItemMessage (hdlg, IDC_SLIDER_LF2, TBM_SETTICFREQ, 0, 0);
        SendDlgItemMessage (hdlg, IDC_SLIDER_LF2, TBM_SETRANGE, false, MAKELONG (0, 100));
        SendDlgItemMessage (hdlg, IDC_SLIDER_LF2, TBM_SETPOS, true, mfd->Level_2);

        SendDlgItemMessage (hdlg, IDC_SHARPEN, TBM_SETTICFREQ, 0, 0);
        SendDlgItemMessage (hdlg, IDC_SHARPEN, TBM_SETRANGE, false, MAKELONG (0, 6));
        SendDlgItemMessage (hdlg, IDC_SHARPEN, TBM_SETPOS, true, mfd->Sharpen);

        CheckDlgButton(hdlg, IDC_NOISE, mfd->Noise?BST_CHECKED:BST_UNCHECKED);
        CheckDlgButton(hdlg, IDC_DISABLE, mfd->Disable?BST_CHECKED:BST_UNCHECKED);
        CheckDlgButton(hdlg, IDC_INTERLACE, mfd->interlace?BST_CHECKED:BST_UNCHECKED);
        CheckDlgButton(hdlg, IDC_SPEED, mfd->speed?BST_CHECKED:BST_UNCHECKED);

        mfd->ifp->InitButton((VDXHWND)GetDlgItem(hdlg, IDPREVIEW));
        //       
        mfd->Old_Level_1=mfd->Level_1;
        mfd->Old_Level_2=mfd->Level_2;
        mfd->Old_Sharpen=mfd->Sharpen;
        mfd->Old_interlace=mfd->interlace;
        mfd->Old_speed=mfd->speed;

        if (mfd->count==1)
        {
            EnableWindow(GetDlgItem(hdlg, IDC_SPEED), FALSE);
        }
        SetDlgItemInt(hdlg, IDC_LABEL, mfd->count , FALSE);
        return TRUE;
        case WM_HSCROLL:
        if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_LF1))
        {
            mfd->Level_1=SendDlgItemMessage (hdlg, IDC_SLIDER_LF1, TBM_GETPOS, 0, 0);
            mfd->ifp->RedoFrame();
        }
        if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_LF2))
        {
            mfd->Level_2=SendDlgItemMessage (hdlg, IDC_SLIDER_LF2, TBM_GETPOS, 0, 0);
            mfd->ifp->RedoFrame();
        }
        if ((HWND) lParam == GetDlgItem(hdlg, IDC_SHARPEN))
        {
            mfd->Sharpen=SendDlgItemMessage (hdlg, IDC_SHARPEN, TBM_GETPOS, 0, 0);
            mfd->ifp->RedoFrame();
        }
        break;
        case WM_COMMAND:
        switch (LOWORD(wParam))
        {
            case ID_OK:
            mfd->Noise= !!IsDlgButtonChecked(hdlg, IDC_NOISE);
            mfd->Level_1=SendDlgItemMessage (hdlg, IDC_SLIDER_LF1, TBM_GETPOS, 0, 0);
            mfd->Level_2=SendDlgItemMessage (hdlg, IDC_SLIDER_LF2, TBM_GETPOS, 0, 0);
            mfd->Sharpen=SendDlgItemMessage (hdlg, IDC_SHARPEN, TBM_GETPOS, 0, 0);
            EndDialog(hdlg, LOWORD(wParam));
            return (INT_PTR)TRUE;
            case IDPREVIEW:
            mfd->ifp->Toggle((VDXHWND)hdlg);
            mfd->ifp->RedoFrame();
            break;
            case IDC_NOISE:
            mfd->Noise= !!IsDlgButtonChecked(hdlg, IDC_NOISE);
            mfd->ifp->RedoFrame();
            break;
            case IDC_DISABLE:
            mfd->Disable = !!IsDlgButtonChecked(hdlg, IDC_DISABLE);
            mfd->ifp->RedoFrame();
            break;
            case IDC_SPEED:
            mfd->speed  = !!IsDlgButtonChecked(hdlg, IDC_SPEED);
            mfd->ifp->RedoFrame();
            break;
            case IDC_INTERLACE:
            mfd->interlace  = !!IsDlgButtonChecked(hdlg, IDC_INTERLACE);
            mfd->ifp->RedoFrame();
            break;
            case ID_CANCEL:
            mfd->Level_1=mfd->Old_Level_1;
            mfd->Level_2=mfd->Old_Level_2;
            mfd->Sharpen=mfd->Old_Sharpen;
            mfd->interlace=mfd->Old_interlace;
            mfd->speed=mfd->Old_speed;
            EndDialog(hdlg, 1);
            return FALSE;
        }
        break;
    }
    return FALSE;
}

int configProc(VDXFilterActivation *fa, const VDXFilterFunctions *ff, VDXHWND hwndParent)
{
    //   
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    mfd->ifp = fa->ifp;
    int res;
    res=DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_FILTER), (HWND)hwndParent,
                       ConfigDlgProc, (LPARAM)mfd);
    return !res;
}
void StringProc(const VDXFilterActivation *fa, const VDXFilterFunctions *ff, char *str)
{
    //        
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    sprintf(str," (F1-%d,F2-%d,S-%d)", mfd->Level_1,mfd->Level_2,mfd->Sharpen);
}
static void script_config(IVDXScriptInterpreter *isi, void *lpVoid, VDXScriptValue *argv, int argc)
{
    //      avisynth
    VDXFilterActivation *fa = (VDXFilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    mfd->Level_1=argv[0].asInt();
    mfd->Level_2=argv[1].asInt();
    mfd->Sharpen=argv[2].asInt();
    mfd->interlace=argv[3].asInt();
    mfd->speed=argv[4].asInt();
}
bool FssProc(VDXFilterActivation *fa, const VDXFilterFunctions *ff, char *buf, int buflen)
{
    //    
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    _snprintf(buf, buflen, "Config(%d,%d,%d,%d,%d)", mfd->Level_1,mfd->Level_2,mfd->Sharpen,mfd->interlace,mfd->speed);
    return true;
}

int StartProc(VDXFilterActivation *fa, const VDXFilterFunctions *ff)
{
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    //           
    mfd->size = fa->src.size;
    if (!(mfd->y_data = new unsigned char [fa->src.size]))
        return 1;
    memset (mfd->y_data, 0, mfd->size);
    return 0;
}

int EndProc(VDXFilterActivation *fa, const VDXFilterFunctions *ff)
{
    //       
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    delete[] mfd->y_data;
    mfd->y_data = NULL;
    return 0;
}